﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Praktikum_2
{
    abstract class Windenergieanlage
    {
        abstract public double BestimmeLeistung_kW(double windgeschwindigkeit);

        public static double Rayleigh(double mittlereWindgeschwindigkeit, double v)
        {
            return 1.0 - Math.Exp(-Math.PI * v * v / 4.0 / (mittlereWindgeschwindigkeit * mittlereWindgeschwindigkeit));
        }

        public double BestimmeJahresenergieertrag_kWh(double mittlereWindgeschwindigkeit)
        {
            double schrittweite = 1.0;
            double jahresenergieertrag = 0.0;
            for (int i = (int)(1.0 / schrittweite); i < 40.01 / schrittweite; i++)
            {
                double v = schrittweite * i;
                double wahrscheinlichkeitDavor = Rayleigh(mittlereWindgeschwindigkeit, v - 0.5 * schrittweite);
                double wahrscheinlichkeitDahinter = Rayleigh(mittlereWindgeschwindigkeit, v + 0.5 * schrittweite);
                jahresenergieertrag += BestimmeLeistung_kW(v) * (wahrscheinlichkeitDahinter - wahrscheinlichkeitDavor) * 24.0 * 365.0;
            }
            return jahresenergieertrag;
        }
    }

    class Maximum : Windenergieanlage
    {
        double rotordurchmesser;

        public Maximum (double rotordurchmesser)
        {
            this.rotordurchmesser = rotordurchmesser;
        }

        public override double BestimmeLeistung_kW(double windgeschwindigkeit)
        {
            return 0.5 * 1.225 * Math.PI * Math.Pow(rotordurchmesser / 2.0, 2) * Math.Pow(windgeschwindigkeit, 3) * 0.001;
        }
    }

    class E126EP4 : Windenergieanlage
    {
        public override double BestimmeLeistung_kW(double windgeschwindigkeit)
        {
            double[,] leistungskurve_kW =
            {
                {2.0, 0.0},
                {3.0, 58.0},
                {4.0, 185.0},
                {5.0, 400.0},
                {6.0, 745.0},
                {7.0, 1200.0},
                {8.0, 1790.0},
                {9.0, 2450.0},
                {10.0, 3120.0},
                {11.0, 3660.0},
                {12.0, 4000.0},
                {13.0, 4150.0},
                {14.0, 4200.0},
                {30.0, 4200.0},
                {31.0, 0.0}
            };

            // GetLength(0) war noch nicht dran.
            if (windgeschwindigkeit <= leistungskurve_kW[0, 0] || windgeschwindigkeit >= leistungskurve_kW[leistungskurve_kW.GetLength(0) - 1, 0])
            {
                return 0.0;
            }

            int nächsterIndex = -1;
            for (int i = 1; i < leistungskurve_kW.GetLength(0); i++)
            {
                if(leistungskurve_kW[i, 0] >= windgeschwindigkeit)
                {
                    nächsterIndex = i;
                    break;
                }
            }

            int vorherigerIndex = nächsterIndex - 1;

            return leistungskurve_kW[vorherigerIndex, 1]
                + (windgeschwindigkeit - leistungskurve_kW[vorherigerIndex, 0])
                  / (leistungskurve_kW[nächsterIndex, 0] - leistungskurve_kW[vorherigerIndex, 0])
                    * (leistungskurve_kW[nächsterIndex, 1] - leistungskurve_kW[vorherigerIndex, 1]);
        }
    }
}
